"""
dgNova - Statistical Analysis and Simulations for Plant Breeding Experiments
"""

__version__ = '0.1.0'

from .designs import REP, UNREP, RCBD, Lattice, AlphaLattice

__all__ = ['REP', 'UNREP', 'RCBD', 'Lattice', 'AlphaLattice'] 

# Copyright (c) 2024 Nadim Khan
# 
# This software is released under the MIT License.
# https://opensource.org/licenses/MIT 