"""
Test suite for dgNova package.

This package contains all test modules for dgNova functionality including:
- Unreplicated designs
- RCBD
- Latin square
- Alpha lattice
- Input/Output operations
"""

import pytest

# Configure pytest
pytest.register_assert_rewrite('numpy.testing')